require 'ratr'
require 'ostruct'

describe Ratr::Source do
  let(:movies) { {
    first: OpenStruct.new(title: "8 Mile", year: "2002"),
    second: OpenStruct.new(title: "The Abyss", year: "1989") }
  }

  describe 'rotten tomatoes source' do
    let(:source) { Ratr::Source.new(Ratr::SerialHttpProcessor, Ratr::RottenTomatoesHttpWrapper.new) }

    describe '#fetch', :vcr do
      it "returns scores for each movie provided" do
        fetched_scores = source.fetch(movies)

        expect(fetched_scores[:first]).to eq([5.4, 7.6])
        expect(fetched_scores[:second]).to eq([8.3, 8.9])
      end
    end
  end

  describe 'omdb source' do
    let(:source) { Ratr::Source.new(Ratr::ParallelHttpProcessor, Ratr::OmdbHttpWrapper.new) }

    describe '#fetch', :vcr do
      it "returns scores for each movie provided" do
        fetched_scores = source.fetch(movies)

        expect(fetched_scores[:first]).to eq [6.9]
        expect(fetched_scores[:second]).to eq [7.6]
      end
    end
  end
end
